"""
SmithTube by ANVF
This generates Loadpull Contours in a SmithTube, with frequency as Z-axis.
It imports a tab separated loadpull file with the following buildup:
vswr	    1	2	3	4	5	6
angle	    21	22	23	24	25	26 (0-360 for each swr)
2440_12_f1	31	32	33	34	35	36
2440_12_f2	41	42	43	44	45	46
2440_12_iav	51	52	53	54	55	56
2440_12_eff	61	62	63	64	65	66
Freq_PL_MeasParam
To generate this file, in Excel remove all other tabs, then file>export>changeFileType>Tab delimited txt. 
 
20210422: First version 
20210723: Mesh changed to linspace. Added variables plotwhat, PL, freqList in example.
20210728: Cleaned up grid in plot_smith (3D). Added functionality to not plot any Smith Chart but still plot in 3D.
          Added alternative plot - plot_rowi_data_alt()
20210729: Added polar plot function to "create_smith_chart_data".
20210730: Color scale improved, chart title added. Darker cross in plot_smith()
"""

import numpy as np
import matplotlib.pyplot as plt
import scipy.interpolate
import os
import csv
import time
# # FUNCTIONS # #

# create_smith_chart_data creates the data for a smith chart with constant resistance circles and constant reactances lines.
# Input:    R1      - Base for the  smith chart
#           polar   - Boolean determining if polar data should be created instead of smith chart data
# Output:   sc      - data containing constant resistance circles
#           sc2     - data containing constant reactance lines
def create_smith_chart_data(R1=50, polar=False):
    sr = np.array([R1*0.2, R1*0.5,R1, R1*2, R1*5, R1*30])
    sx = np.array(range(-1000,1000))*1j
    if polar==False:
        for i in range(0,len(sr)):
            if i==0:
                sz = sr[i] + sx
            else:
                sz = np.vstack((sz,sr[i] +sx))
        sr2 = np.array(range(0,10000))
        sx2 = np.array([0, 0.2*R1, 0.5*R1, R1, 2*R1, 5*R1, 30*R1, -0.2*R1, -0.5*R1, -R1, -2*R1, -5*R1, -30*R1])*1j
        for i in range(0,len(sx2)):
            if i==0:
                sz2 = sx2[i] + sr2
            else:
                sztemp = sx2[i]+sr2
                sz2 = np.vstack((sz2,sztemp))
        sc = (sz - R1)/ (sz + R1)
        sc2 = (sz2 - R1) / (sz2 + R1)
    if polar==True:
        phase = np.arange(0,360,0.1) * np.pi / 180
        Gamma = [0.2,0.4,0.6, 0.8]
        for i in range(0,len(Gamma)):
            if i == 0:
                Gamma_r = Gamma[i] * np.cos(phase)
                Gamma_i = Gamma[i] * np.sin(phase)
            else:
                Gamma_temp = Gamma[i] * np.cos(phase)
                Gamma_r = np.vstack((Gamma_r,Gamma_temp))
                Gamma_temp = Gamma[i] * np.sin(phase)
                Gamma_i = np.vstack((Gamma_i,Gamma_temp))
        sc = Gamma_r + 1j*Gamma_i
        sc2 = []
    return sc, sc2




# plot_smith plots a 3D smith chart. Needs plt.show() to actually show anything.
# Inputs: sc    - Complex data containing constant resistance circles
#         sc2   - Complex data containing constant reactance lines
#         k     - Choose at what level level you want to plot a smith chart / polar plot
# Output: fig   - variable containing the figure
#         ax    - variable containing what to be plotted
def plot_smith(sc=[], sc2=[], k=[1]):
    fig = plt.figure()
    ax = plt.axes(projection='3d')
    ax.xaxis._axinfo["grid"].update({'linewidth':0.5,'color':'#A5A5A5'})
    ax.yaxis._axinfo["grid"].update({'linewidth':0.5,'color':'#A5A5A5'})
    ax.zaxis._axinfo["grid"].update({'linewidth':0.5,'color':'#A5A5A5'})
    ax.xaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
    ax.yaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
    ax.zaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
    if k!=None:
        for j in range(0,len(k)):
            theta = np.arange(0, 2*np.pi, np.pi/1000)
            X = np.cos(theta)
            Y = np.sin(theta)
            Z = X/X
            ax.plot3D(X,Y,Z*k[j],'gray') # outer circle
            if len(sc) != 0:
                X = np.real(sc)
                Y = np.imag(sc)
                Z = np.ones_like(np.imag(sc))
                for i in range(0,len(X)):
                    ax.plot3D(X[i],Y[i],Z[i]*k[j], '#A5A5A5') # constant Zreal lines
            if len(sc2) != 0:
                X = np.real(sc2)
                Y = np.imag(sc2)
                Z = np.ones_like(np.real(sc2))
                for i in range(0,len(X)):
                    ax.plot3D(X[i],Y[i],Z[i]*k[j], '#A5A5A5') # constant Zimag lines
                    
            # the following make two lines with different colour, to mark the origin.
            # you can comment out if not needed,
            X = np.arange(-1,1,0.01) # GammaImag=0 line
            Y = X*0
            Z = X/X   
            ax.plot3D(X,Y,Z*k[j], 'darkslategrey') #
            Y = np.arange(-1,1,0.01) # GammaReal=0 line
            X = Y*0
            Z = Y/Y   
            ax.plot3D(X,Y,Z*k[j], 'darkslategrey') #
            
    return fig, ax

# load_rowi_data loads a text file given in the format as default file_path suggest.
# Inputs:   file_path   - string with text file containing data.
# Outputs:  header      - contains the title for each line of data.
#           data        - contains the data corresponding to each header line.
def load_rowi_data(file_path='PanaLab4_module_loadpull_20210408_EffExport.txt'):
    #file_path = 'panaLab4_module_loadp_20210326_export.txt'
    txt = np.loadtxt(file_path, delimiter='\t', skiprows=0, dtype=str)
    if txt[0,len(txt[0,:])-1] == '':
        txt = np.delete(txt,len(txt[0,:])-1,1)
    header = txt[:,0]
    data = txt[:,1:].astype(float)
    return header, data

# plot_rowi_data plots the specified data on a 3D smith chart.
# Inputs: header            - contains the title for each line of data
#         data              - contains the data corresponding to each header line
#         ax                - variable containing what to be plotted. (use the ax from "plot_smith" as input)
#         plotwhat          - string instructing what to be plotted. f1, f2, iav, eff
#         PL                - string instructing what PL to plot
#         freq              - string instructing what frequency to plot
#         Contourlevels     - list of ints instructing what contourlevels to show
# Outputs: ax               - variable containing what to be plotted.
def plot_rowi_data(header, data, ax, plotwhat='f1', PL='12', freq='2404', ContourLevels = None):
    freq_list = np.arange(2402,2482,2).astype(str)
    freq_index = np.where(freq_list==freq)[0][0]

    swr_index = np.where(header=='swr')[0][0]
    angle_index = np.where(header=='angle')[0][0]
    swr = data[swr_index,:]
    phase = data[angle_index,:]*np.pi/180
    Gamma = (swr-1) / (swr+1)
    x = Gamma * np.cos(phase)
    y = Gamma * np.sin(phase)

    if plotwhat == 'f1':
        name = freq + '_' + PL + '_f1'
        data_index = np.where(header==name)[0][0]
        z = data[data_index,:]
    elif plotwhat == 'f2':
        name = freq + '_' + PL + '_f2'
        data_index = np.where(header==name)[0][0]
        z = data[data_index,:]
    elif plotwhat == 'iav':
        name = freq + '_' + PL + '_iav'
        data_index = np.where(header==name)[0][0]
        z = data[data_index,:]
    elif plotwhat == 'eff':
        name = freq + '_' + PL + '_eff'
        data_index = np.where(header==name)[0][0]
        z = data[data_index,:]
    else:
        print("Information not found or not collected.")
        return 0
    
    # print(z) #debug only
    ContourLevels=list(np.linspace(z.min(),z.max(),num=13)) # number of contourlines
    
    Ax = np.linspace(x.min(),x.max(),50)
    Ay = np.linspace(y.min(),y.max(),50)
    #xgrid, ygrid = np.meshgrid(Ax, Ay)
    
    xi, yi = np.meshgrid(Ax, Ay)
    zi = scipy.interpolate.griddata((x, y), z, (xi, yi), method='cubic')

    # ngridx = 2000
    # ngridy = 2000
    # xi = np.linspace(-1, 1, ngridx)
    # yi = np.linspace(-1, 1, ngridy)
    # zi = scipy.interpolate.griddata((x, y), z, (xi[None,:], yi[:, None]), method='cubic')

    # ax.contour(xi, yi, zi, levels=[15,17.5,20])
    ax_dude = ax.contour(xi, yi, zi, zdir='z', offset=freq_index, levels = ContourLevels, colors=colorScale)
    # ax.contour(xi, yi, zi, zdir='z', offset=0, levels=clevels)
    # ax.contour(xi, yi, zi, zdir='z', offset=39, levels=clevels)
    return ax_dude

# plot_rowi_data_alt plots data directly as they are collected.
# Inputs: header            - contains the title for each line of data
#         data              - contains the data corresponding to each header line
#         ax                - variable containing what to be plotted. (use the ax from "plot_smith" as input)
#         plotwhat          - string instructing what to be plotted. f1, f2, iav, eff
#         PL                - string instructing what PL to plot
#         freq              - string instructing what frequency to plot
#         Contourlevels     - list of ints instructing what contourlevels to show
# Outputs: ax               - variable containing what to be plotted.
def plot_rowi_data_alt(header, data,ax, plotwhat='f1', PL='12', freq='2404'):
    freq_list = np.arange(2402, 2482, 2).astype(str)
    freq_index = np.where(freq_list == freq)[0][0]
    swr_index = np.where(header == 'swr')[0][0]
    angle_index = np.where(header == 'angle')[0][0]
    swr = data[swr_index, :]
    phase = data[angle_index, :] * np.pi / 180
    Gamma = (swr - 1) / (swr + 1)
    x = Gamma * np.cos(phase)
    y = Gamma * np.sin(phase)
    if plotwhat == 'f1':
        name = freq + '_' + PL + '_f1'
        data_index = np.where(header == name)[0][0]
        z = data[data_index, :]
    elif plotwhat == 'f2':
        name = freq + '_' + PL + '_f2'
        data_index = np.where(header == name)[0][0]
        z = data[data_index, :]
    elif plotwhat == 'iav':
        name = freq + '_' + PL + '_iav'
        data_index = np.where(header == name)[0][0]
        z = data[data_index, :]
    elif plotwhat == 'eff':
        name = freq + '_' + PL + '_eff'
        data_index = np.where(header == name)[0][0]
        z = data[data_index, :]
    else:
        print("Information not found or not collected.")
        return 0

    
    ax_dude = ax.plot(x, y, z, marker='o', label=freq)
    return ax_dude


#### Example how to execute script ############################################

plotwhat='f1' # choose from 'f1'. 'f2', 'iav', 'eff'
PL='12'
freqList=['2404','2440','2478']
# freqList=['2402','2440','2480']
alt_plot = False # alt is 3D with Pout as Z-axis
polarInsteadOfSmith= False #True

fileName= 'panaLab4_module_loadp_20210326_export.txt'
fileName= 'PanaLab4_BAW_loadpull_20210408_export.txt'
fileName= 'PanaLab4_module_loadpull_20210408_EffExport.txt'
# fileName= 'Amber2_Lab2_match1_sn69189765.txt'
# fileName= 'Amber2_lab2_match2_sn69186813.txt'
# fileName= 'Amber_67490535_loadpull_RF4.txt'

# plot contours if the parameter is available in the loadpull file.
# NB use a colorScale with the same number of values as the contourLevels.
# colorScale=['#0610c7', '#1307f5', '#0566f7', '#05c5f5','#05f772', '#07f50f', '#60f205', '#c5f505', '#f7f305','#fac505', '#f76605', '#fa0905', '#cc060d']
colorScale=['navy', 'blue', 'royalblue', 'dodgerblue','deepskyblue', 'aqua', 'springgreen', 'lime','gold','orange', 'tomato', 'red', 'crimson']
# NB for colors see https://matplotlib.org/stable/gallery/color/named_colors.html


if alt_plot==False:
    # Start by creating the smith chart
    sc, sc2 = create_smith_chart_data(polar=polarInsteadOfSmith)
    # Create the smith chart 3D plot. Here I use 0, 19, 39 as the channels I want the smith chart to appear at.
    fig, ax = plot_smith(sc, sc2, k=[0,19,39])

    # load the data from the txt file.
    head, dat = load_rowi_data(file_path=fileName)

    # Plot the loaded data. I use the average current, Power Lvl 12 and 3 different frequencies (channel 1, 19, 38)
    plot_rowi_data(head, dat, ax, plotwhat=plotwhat, PL=PL, freq=freqList[1])  # ,ContourLevels=[12,15,16])
    plot_rowi_data(head, dat, ax, plotwhat=plotwhat, PL=PL, freq=freqList[2])  # ,ContourLevels=[12,15,16])
    lastax = plot_rowi_data(head, dat, ax, plotwhat=plotwhat, PL=PL, freq=freqList[0])  # ,ContourLevels=[12,15,16])
    # NOTE: lastax is used to create the colorbar.
    # NOTE: Add ContourLevels if you want specific contours to be shown. (The lines has to be represented by the data).
    ax.set_zlim3d([0,40])
    fig.colorbar(lastax, shrink=0.5, aspect=5)
    ax.set_zlabel('Channel No.')
    ax.set_title('Loadpull contours vs channels(z-axis), PL'+PL, fontsize=8)
elif alt_plot==True: # alt is 3D with Pout as Z-axis
    # Start by creating the smith chart
    sc, sc2 = create_smith_chart_data()
    # Create the smith chart 3D plot. Here I use 0, 19, 39 as the channels I want the smith chart to appear at.
    fig, ax = plot_smith(sc=None, sc2=None, k=None)

    # load the data from the txt file.
    head, dat = load_rowi_data(file_path=fileName)

    # Plot the loaded data. I use the average current, Power Lvl 12 and 3 different frequencies (channel 1, 19, 38)
    plot_rowi_data_alt(head, dat, ax, plotwhat=plotwhat, PL=PL, freq=freqList[0])
    plot_rowi_data_alt(head, dat, ax, plotwhat=plotwhat, PL=PL, freq=freqList[1])
    plot_rowi_data_alt(head, dat, ax, plotwhat=plotwhat, PL=PL, freq=freqList[2])
    ax.legend()
    ax.set_zlabel('dBm')
    ax.set_title('3D Loadpull contours, PL'+PL, fontsize=8)
    # ax.set_zlim(-1,10)

ax.set_xlabel('$\Gamma_r$')
ax.set_ylabel('$\Gamma_i$')


plt.show()





# # OLD STUFF # #
'''
datmat = ["C:\\Users\\ANVF\\Downloads\\SKY66403\\SKY66403\SKY66403-11 HEM Vcc=3p0 Pin=0dbm_2402.0MHz-COPY.txt"]
def plot_data_OLD(data_file,ax,channel=19,clevels =[15, 17.5, 20],plotwhat="Pout"):
    data = np.loadtxt(data_file[0], delimiter='\t', skiprows=13, dtype=str)
    Gamma = data[:, 1].astype(float)
    Phase = data[:, 2].astype(float)
    Pin = data[:, 4].astype(float)
    Pout = data[:, 5].astype(float)
    Gain = data[:, 6].astype(float)
    Icc1 = data[:, 7].astype(float)
    Icc2 = data[:, 8].astype(float)
    P_DC = (3*(Icc1+Icc2)*1E-3) # Replace 3 with your Voltage.

    x = Gamma * np.cos(Phase * np.pi / 180)
    y = Gamma * np.sin(Phase * np.pi / 180)
    if plotwhat == "Pout":
        z = Pout
    elif plotwhat =="PAE":
        z = (np.power(10,Pout/10)/1000 - np.power(10,Pin/10)/1000)/P_DC # PAE
    elif plotwhat == "Drain":
        z = (np.power(10,Pout/10)/1000)/P_DC
    elif plotwhat == "Icc":
        z = Icc2;
    xi, yi = np.meshgrid(x, y)
    zi = scipy.interpolate.griddata((x, y), z, (xi, yi), method='cubic')

    # ngridx = 2000
    # ngridy = 2000
    # xi = np.linspace(-1, 1, ngridx)
    # yi = np.linspace(-1, 1, ngridy)
    # zi = scipy.interpolate.griddata((x, y), z, (xi[None,:], yi[:, None]), method='cubic')

    # ax.contour(xi, yi, zi, levels=[15,17.5,20])
    x = ax.contour(xi, yi, zi, zdir='z', offset=channel, levels=clevels)
    # ax.contour(xi, yi, zi, zdir='z', offset=0, levels=clevels)
    # ax.contour(xi, yi, zi, zdir='z', offset=39, levels=clevels)
    return x
'''