# -*- coding: utf-8 -*-
"""
loadpull_contours_yyyymmdd.py by ROWI@≡Apollo
D:\python\WPy64-3940

This generates Loadpull Contours in a 2D gamma-plane.
A separate contour plot for each available MeasParam is created, at the selected PL and Freq.
SmithChart can be added under the contour (set add_smithchart= True).
Alternatively, the loadpull contours can be plotted in 3D on top of the SmithChart.
You can also choose to use a Polar chart instead of a SmitchChart.
It imports a tab separated loadpull file with the following buildup:
vswr	    1	2	3	4	5	6
angle	    21	22	23	24	25	26 (0-360 for each swr)
2440_12_f1	31	32	33	34	35	36
2440_12_f2	41	42	43	44	45	46
2440_12_iav	51	52	53	54	55	56
2440_12_eff	61	62	63	64	65	66
Freq_PL_MeasParam
To generate this file, in Excel remove all other tabs, then file>export>changeFileType>Tab delimited txt. 


20210127: first version, with Gamma re/im scale
20210128: contour autoscale added (should be changed so we get nice values instead of fixed number of lines)
20210201: 13 different colors defined. Colorbar added.
20210723: Smithchart added, courtesy of ANVF. Can be switched off with the variable 'add_smithchart''
20210726: common colorScale-list for all plots; very light colors avoided.
20210729: change from Smith to Polar with 'use_polar_not_smith'
20211110: change x and y scales of the 'plot without smithchart' from auto to [-1,1]. 
20211111: 'WidthFactor' variable added, to correct aspect ratio when colorbar is used.

#todo: change console text when some data is not available
"""

import os
import time
import csv
import numpy as np
import matplotlib.pyplot as plt
from scipy.interpolate import griddata
#from mpl_toolkits.mplot3d import Axes3d # is not available

script_dir = os.getcwd() # gets current working directory
print('current working directory',script_dir)
print('files in this directory: ',os.listdir()) # lists all files in this directory
# NB file path is relative to working directory

##### user inputs: >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

#file_path = 'PanaLab4_loadpull_3freq3PL_horizontal_20210127_2003.txt'
# file_path = 'panaLab4_module_loadp_20210326_export.txt'
#file_path = 'PanaLab4_loadpull_harm_20210401_2200.txt'
file_path = 'PanaLab4_module_loadpull_20210408_EffExport.txt'
file_path = 'PanaLab4_BAW_loadpull_20210408_export.txt'
# file_path = 'PanaLab4_module_harm_loadpull_20210408_export.txt'
# file_path = 'Amber_67490536_loadpull_RF4.txt' # Amber2 lab1 DUT1
# file_path = 'Amber_67490535_loadpull_RF4.txt'
# file_path = 'Amber2_lab2_match2_sn69186813.txt' # Amber2 lab2 DUT2 with L,L,C match
# file_path = 'amber2_Lab2_match1_sn69189765.txt' # Amber2 lab2 DUT2 with C,L,C match

WidthFactor = 1.22 # factor of 1.22 (default is 1) is needed to stay square when colorbar is used.
# NB this factor shall be changed to e.g. 1.25 when concole graphics is set to Automatic instead of Inline. 

# SELECT A POWERLEVEL, FREQUENCY, PLOT-MODE:
PL='12'
#PL='11'
# PL='9'
freq='2402'
# freq='2404'
freq='2440'
#freq='2478'
# freq='2480'
harm_loadpull=  False #True # will only plot the harmonics, with updated text

add_smithchart= False #True # adds smithchart below the contour plot.
add_smithchart3D= False #True # adds smithchart below the contour plot, and plots the contours in 3D.
use_polar_not_smith= False #True # uses polar chart instead of smithchart.

#### end of user inputs <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


# create_smith_chart_data creates the data for a smith chart with constant resistance circles and constant reactances lines.
# Input:    R1      - Base for the  smith chart
#           polar   - uses Polar chart instead of SmithChart
# Output:   sc      - data containing constant resistance circles
#           sc2     - data containing constant reactance lines
def create_smith_chart_data(R1=50, polar=False): 
    sr = np.array([R1*0.2, R1*0.5,R1, R1*2, R1*5, R1*30])
    sx = np.array(range(-1000,1000))*1j
    if polar==False:
        for i in range(0,len(sr)):
            if i==0:
                sz = sr[i] + sx
            else:
                sz = np.vstack((sz,sr[i] +sx)) # x,y of reactance lines with 2000points for each resistance sr
        sr2 = np.array(range(0,10000))
        sx2 = np.array([0, 0.2*R1, 0.5*R1, R1, 2*R1, 5*R1, 30*R1, -0.2*R1, -0.5*R1, -R1, -2*R1, -5*R1, -30*R1])*1j
        for i in range(0,len(sx2)):
            if i==0:
                sz2 = sx2[i] + sr2
            else:
                sztemp = sx2[i]+sr2
                sz2 = np.vstack((sz2,sztemp)) # x,y of resistance lines with 10000 points for each reactance sx2
        sc = (sz - R1)/ (sz + R1) # gamma of the reactance lines is (z-z0)/(z+z0)
        sc2 = (sz2 - R1) / (sz2 + R1) # gamma of the resistance lines is (z-z0)/(z+z0)
    if polar==True:
        phase = np.arange(0,360,0.1) * np.pi / 180
        Gamma = [0.2,0.4,0.6, 0.8]
        for i in range(0,len(Gamma)):
            if i == 0:
                Gamma_r = Gamma[i] * np.cos(phase)
                Gamma_i = Gamma[i] * np.sin(phase)
            else:
                Gamma_temp = Gamma[i] * np.cos(phase)
                Gamma_r = np.vstack((Gamma_r,Gamma_temp))
                Gamma_temp = Gamma[i] * np.sin(phase)
                Gamma_i = np.vstack((Gamma_i,Gamma_temp))
        sc = Gamma_r + 1j*Gamma_i
        sc2 = []
    return sc, sc2

def plot_smith2d(sc, sc2):
    fig, ax = plt.subplots(figsize=(8*1.25, 8)) # 1.25 needed when colorbar is added
    theta = np.arange(0, 2*np.pi, np.pi/1000)
    X = np.cos(theta)
    Y = np.sin(theta)
    ax.plot(X,Y,'gray')
    # X = np.arange(-1,1,0.01) # not needed after 0 was added in sx2
    # Y = X*0
    # ax.plot(X,Y,'lightgray', zorder=1) # line where reactance=0
    X = np.real(sc)
    Y = np.imag(sc)
    for i in range(0,len(X)): 
        ax.plot(X[i],Y[i], 'lightgray', linewidth=2, zorder=1) # constant resistance lines
    X = np.real(sc2)
    Y = np.imag(sc2)
    Z = np.ones_like(np.real(sc2)) # makes array of ones with same shape
    for i in range(0,len(X)):
        ax.plot(X[i],Y[i], 'lightgray', linewidth=2, zorder=1) # constant reactance lines
    return fig, ax

# plot_smith3d plots a 3D smith chart. Needs plt.show() to actually show anything.
# Inputs: sc    - Complex data containing constant resistance circles
#         sc2   - Complex data containing constant reactance lines
# Output: fig   - variable containing the figure
#         ax    - variable containing what to be plotted
def plot_smith3d(sc, sc2, k=[1]):
    fig = plt.figure()
    ax = plt.axes(projection='3d')
    ax.xaxis._axinfo["grid"].update({'linewidth':0.5,'color':'#A5A5A5'})
    ax.yaxis._axinfo["grid"].update({'linewidth':0.5,'color':'#A5A5A5'})
    ax.zaxis._axinfo["grid"].update({'linewidth':0.5,'color':'#A5A5A5'})
    ax.xaxis.set_pane_color((1.0, 1.0, 1.0, 0.0)) # R,G,B,translucency
    ax.yaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
    ax.zaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
    for j in range(0,len(k)):
        theta = np.arange(0, 2*np.pi, np.pi/1000)
        X = np.cos(theta)
        Y = np.sin(theta)
        Z = X/X
        ax.plot3D(X,Y,Z*k[j],'gray')
        X = np.real(sc)
        Y = np.imag(sc)
        Z = np.ones_like(np.real(sc))
        for i in range(0,len(X)):
            ax.plot3D(X[i],Y[i],Z[i]*k[j], 'gray') # constant resistance lines
        X = np.real(sc2)
        Y = np.imag(sc2)
        Z = np.ones_like(np.real(sc2))
        for i in range(0,len(X)):
            ax.plot3D(X[i],Y[i],Z[i]*k[j], 'red')#'gray')
    return fig, ax


with open(file_path, 'r') as f:
    data = list(csv.reader(f, delimiter="\t")) # is a list of lists(are the lines with all its values)
#print(data) 
lines=len(data)
#print(lines)

pf1=[]
pf2=[]
iav=[]
eff=[]

for l in range(len(data)): # every list (line) seperately
    #print(l)
    line=data[l] # one line only
#    print(line)
    name=line[0] # first string in each line
    print(name+ '=', end=' ')
    if line[-1]=='': line=line[:-1] # remove the last if empty string (when txtfile lines ended with tab)
    if name== 'vswr' or name=='swr':
        swr= np.array(line[1:], dtype=float)
        print(swr)  
    elif name== 'angle':
        angle= np.array(line[1:], dtype=float)
#        angle= np.array(line[1:]).astype(float)
        print(angle)
    elif name== freq+'_'+PL+'_f1':
        pf1= np.array(line[1:], dtype=float)
        print(pf1)
    elif name== freq+'_'+PL+'_f2':
        pf2= np.array(line[1:], dtype=float)
        print(pf2)
    elif name== freq+'_'+PL+'_iav':
        iav= np.array(line[1:], dtype=float) 
        print(iav)
    elif name== freq+'_'+PL+'_eff':
        eff= np.array(line[1:], dtype=float) 
        print(eff)
    else: print('available, but not selected')
       
#gamma_mag = (swr-1)/(swr+1)
gamma_mag = np.clip((swr-1)/(swr+1),1e-3, .99) # limit value to min and max
gamma_r = gamma_mag*np.cos(angle*np.pi/180) # real
gamma_i = gamma_mag*np.sin(angle*np.pi/180) # imaginary
R = 50*(1-gamma_r**2-gamma_i**2)/((1-gamma_r)**2+gamma_i**2) # Z_real
X = 50*(2*gamma_i)/((1-gamma_r)**2+gamma_i**2) # Z_imaginary

# create uniform XY grid for Load Gamma:
Ax = np.linspace(gamma_r.min(),gamma_r.max(),50)
Ay = np.linspace(gamma_i.min(),gamma_i.max(),50)
xgrid, ygrid = np.meshgrid(Ax, Ay)

# Start by creating the smith chart
sc, sc2 = create_smith_chart_data(polar=use_polar_not_smith)

# plot contours if the parameter is available in the loadpull file.
# NB use a colorScale with the same number of values as the contourScale.
# colorScale=['#0610c7', '#1307f5', '#0566f7', '#05c5f5','#05f772', '#07f50f', '#60f205', '#c5f505', '#f7f305','#fac505', '#f76605', '#fa0905', '#cc060d']
colorScale=['navy', 'blue', 'royalblue', 'dodgerblue','deepskyblue', 'aqua', 'springgreen', 'lime','gold','orange', 'tomato', 'red', 'crimson']
# NB for colors see https://matplotlib.org/stable/gallery/color/named_colors.html
        
if harm_loadpull!=True:

    #Pout
    param=pf1[:]
    if np.size(param)!=0:
        contourScale=list(np.linspace(param.min(),param.max(),num=13)) # number of contourlines
        zgrid = griddata((gamma_r,gamma_i),param, (xgrid, ygrid), method='cubic')#'linear') # 'nearest') #, fill_value=0) # default fill is nan, which can give problems.
        #print(Ax)
        #print(Ay)
        #print(zgrid)
        if add_smithchart3D:
            fig, chart= plot_smith3d(sc,sc2) # this plots the 3D contour inside the figure with the smithchart
        elif add_smithchart:
            fig, chart= plot_smith2d(sc,sc2) # this plots the contour inside the figure with the smithchart
        else:
            fig, chart = plt.subplots(figsize=(8*WidthFactor, 8)) # 1.22 needed when colorbar is added
        
        #CS=chart.contour(xgrid,ygrid,zgrid,levels=[2,2.5,3,3.5,4,4.5,5,5.5,6,6.5,7,7.5])
        # CS=chart.contour(xgrid,ygrid,zgrid,levels=contourScale)
        CS=chart.contour(xgrid,ygrid,zgrid,levels=contourScale, colors=colorScale, extend='both')
        chart.clabel(CS, inline=1, fontsize=10)
        fig.colorbar(CS)
        chart.set_title('Loadpull Pout contour for '+freq+'MHz, PL'+PL)
        chart.grid(True)
        chart.set_xlabel('gamma_real')
        chart.set_ylabel('gamma_imag')
        chart.set_xlim(-1,1) #
        chart.set_ylim(-1,1) # 
        plt.show()
        
    
    #Iavg
    param=iav[:]
    if np.size(param)!=0:
        #print(param.min(),param.max())
        contourScale=list(np.linspace(param.min(),param.max(),num=13)) # number of contourlines
        zgrid = griddata((gamma_r,gamma_i),param, (xgrid, ygrid), method='cubic')#, fill_value=0) # default fill is nan, which can give problems.      
        if add_smithchart3D:
            fig, chart= plot_smith3d(sc,sc2)
        elif add_smithchart:
            fig, chart= plot_smith2d(sc,sc2)
        else:
            fig, chart = plt.subplots(figsize=(8*WidthFactor, 8)) # 1.22 needed when colorbar is added     
            
        #CS=chart.contour(xgrid,ygrid,zgrid,levels=[8,8.25,8.5,8.75,9,9.25,9.5,9.75,10,10.25,10.5,10.75, 11, 11.25, 11.5])
        CS=chart.contour(xgrid,ygrid,zgrid,levels=contourScale, colors=colorScale, extend='both')
        chart.clabel(CS, inline=1, fontsize=10)
        fig.colorbar(CS)
        chart.set_title('Loadpull Iavg contour for '+freq+'MHz, PL'+PL)
        chart.grid(True)
        chart.set_xlabel('gamma_real')
        chart.set_ylabel('gamma_imag')
        chart.set_xlim(-1,1) #
        chart.set_ylim(-1,1) #
        plt.show()

    #PA_efficiency
    param=eff[:]
    if np.size(param)!=0:
        #print(param.min(),param.max())
        contourScale=list(np.linspace(param.min(),param.max(),num=13)) # number of contourlines
        zgrid = griddata((gamma_r,gamma_i),param, (xgrid, ygrid), method='cubic')#, fill_value=0) # default fill is nan, which can give problems.
        if add_smithchart3D:
            fig, chart= plot_smith3d(sc,sc2)
        elif add_smithchart:
            fig, chart= plot_smith2d(sc,sc2)
        else:
            fig, chart = plt.subplots(figsize=(8*WidthFactor, 8)) # 1.22 needed when colorbar is added
        CS=chart.contour(xgrid,ygrid,zgrid,levels=contourScale, colors=colorScale, extend='both')
        chart.clabel(CS, inline=1, fontsize=10)
        fig.colorbar(CS)
        chart.set_title('Loadpull PA_eff contour for '+freq+'MHz, PL'+PL)
        chart.grid(True)
        chart.set_xlabel('gamma_real')
        chart.set_ylabel('gamma_imag')
        chart.set_xlim(-1,1) #
        chart.set_ylim(-1,1) #
        plt.show()    
        
    #2ndHarm
    param=pf2[:]
    if np.size(param)!=0:
        #print(param.min(),param.max())
        contourScale=list(np.linspace(param.min(),param.max(),num=13)) # number of contourlines
        zgrid = griddata((gamma_r,gamma_i),param, (xgrid, ygrid), method='cubic')#, fill_value=0) # default fill is nan, which can give problems.
        if add_smithchart3D:
            fig, chart= plot_smith3d(sc,sc2)
        elif add_smithchart:
            fig, chart= plot_smith2d(sc,sc2)
        else:
            fig, chart = plt.subplots(figsize=(8*WidthFactor, 8)) # 1.22 needed when colorbar is added
        CS=chart.contour(xgrid,ygrid,zgrid,levels=contourScale, colors=colorScale, extend='both')
        chart.clabel(CS, inline=1, fontsize=10)
        fig.colorbar(CS)
        chart.set_title('Loadpull 2ndHarm contour for '+freq+'MHz, PL'+PL)
        chart.grid(True)
        chart.set_xlabel('gamma_real')
        chart.set_ylabel('gamma_imag')
        chart.set_xlim(-1,1) #
        chart.set_ylim(-1,1) #
        plt.show()
    

if harm_loadpull==True:
    freq_int=np.int(freq)
    
    #2ndHarm
    param=pf2[:]
    #print(param.min(),param.max())
    contourScale=list(np.linspace(param.min(),param.max(),num=13)) # number of contourlines
    zgrid = griddata((gamma_r,gamma_i),param, (xgrid, ygrid), method='cubic')#, fill_value=0) # default fill is nan, which can give problems.
    if add_smithchart:
        fig, chart= plot_smith2d(sc,sc2)
    elif add_smithchart3D:
        fig, chart= plot_smith3d(sc,sc2)
    else:
        fig, chart = plt.subplots(figsize=(8*WidthFactor, 8)) # 1.22 needed when colorbar is added
    CS=chart.contour(xgrid,ygrid,zgrid,levels=contourScale, colors=colorScale, extend='both')
    chart.clabel(CS, inline=1, fontsize=10)
    fig.colorbar(CS)
    chart.set_title('Harmonic Loadpull 2ndHarm contour for '+freq+'MHz, PL'+PL)
    chart.grid(True)
    chart.set_xlabel('gamma_real')
    chart.set_ylabel('gamma_imag')
    chart.set_xlim(-1,1) #
    chart.set_ylim(-1,1) #
    plt.show()
    